<div class="btn-group sodar-list-btn-group pull-right">
  <button class="btn btn-secondary dropdown-toggle sodar-list-dropdown"
      type="button" data-toggle="dropdown" aria-expanded="false">
    <i class="iconify" data-icon="mdi:cog"></i>
  </button>
  <div class="dropdown-menu dropdown-menu-right">
    <a class="dropdown-item text-danger" href="{% url 'tokens:delete' pk=item.pk %}">
      <i class="iconify" data-icon="mdi:close-thick"></i> Delete Token
    </a>
  </div>
</div>
